﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Kresleni
{
    public partial class Form1 : Form
    {
        BindingList<Primitive> primitives = new BindingList<Primitive>();
        Primitive newPrimitive;
        Color activeColor = Color.Red;
        String activePrimitive = "Kresleni.Line";

        public Form1()
        {
            InitializeComponent();
            DoubleBuffered = true;
            Paint += Form1_Paint;
            MouseDown += Form1_MouseDown;
            MouseMove += Form1_MouseMove;
            MouseUp += Form1_MouseUp;
            
            listBox1.DataSource = primitives;
            listBox1.SelectedIndexChanged += listBox1_SelectedIndexChanged;

            propertyGrid1.PropertyValueChanged += propertyGrid1_PropertyValueChanged;

            primitives.ListChanged += primitives_ListChanged;
        }

        void primitives_ListChanged(object sender, ListChangedEventArgs e)
        {
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
            propertyGrid1.SelectedObject = listBox1.SelectedItem;
        }

        void propertyGrid1_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            Invalidate();
        }

        void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            propertyGrid1.SelectedObject = listBox1.SelectedItem;
        }

        void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            if (newPrimitive != null)
            {
                newPrimitive.BottomRight = new Point(e.X, e.Y);
                primitives.Add(newPrimitive);
                newPrimitive = null;
                Invalidate();
            }
        }

        void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            if (newPrimitive != null)
            {
                newPrimitive.BottomRight = new Point(e.X, e.Y);
                Invalidate();
            }
        }

        void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            newPrimitive = (Primitive)Activator.CreateInstance("Kresleni", activePrimitive).Unwrap();
            newPrimitive.Color = activeColor;
            newPrimitive.LeftTop = new Point(e.X, e.Y);
            newPrimitive.BottomRight = new Point(e.X, e.Y);
        }

        void Form1_Paint(object sender, PaintEventArgs e)
        {
            if (newPrimitive != null) newPrimitive.Draw(e.Graphics);
            Draw(e.Graphics);
        }

        void Draw(Graphics g)
        {
            foreach (Primitive primitive in primitives)
            {
                primitive.Draw(g);
            }
        }

        private void ChangeColor(object sender, MouseEventArgs e)
        {
            activeColor = (sender as Panel).BackColor;
        }

        private void PrimitiveChange(object sender, EventArgs e)
        {
            activePrimitive = "Kresleni."+(sender as ToolStripDropDownItem).Text;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Image img = new Bitmap(ClientSize.Width, ClientSize.Height);
            Graphics g = Graphics.FromImage(img);
            Draw(g);
            img.Save("pokus.bmp");
        }
    }

    public abstract class Primitive
    {
        public int Widht { get; set; }
        public Color Color { get; set; }
        public Point LeftTop { get; set; }
        public Point BottomRight { get; set; }

        public abstract void Draw(Graphics g);
    }

    public class Line : Primitive
    {
        public override void Draw(Graphics g)
        {
            Pen pen = new Pen(Color, Widht);
            g.DrawLine(pen, LeftTop, BottomRight);
        }
    }

    public class Rectangle : Primitive
    {
        public override void Draw(Graphics g)
        {
            Pen pen = new Pen(Color, Widht);

            Point lt = new Point();
            lt.X = Math.Min(LeftTop.X, BottomRight.X);
            lt.Y = Math.Min(LeftTop.Y, BottomRight.Y);

            Size size = new Size();
            size.Width = Math.Abs(LeftTop.X - BottomRight.X);
            size.Height = Math.Abs(LeftTop.Y - BottomRight.Y);

            g.DrawRectangle(pen, new System.Drawing.Rectangle(lt,size));
        }
    }
}
